@@ -8,7 +8,7 @@ Internal functions called from NameParse.pm module
 
 =head1 DESCRIPTION
 
-Grammar tree of personal name syntax for Lingua::EN::NameParse module.
+Grammar tree of personal name syntax for <Lingua::EN::NameParse> module.
 
 The grammar defined here is for use with the Parse::RecDescent module.
 Note that parsing is done depth first, meaning match the shortest string first.
@@ -23,13 +23,10 @@ NameParse::Grammar was written by Kim Ryan <kimryan at cpan dot org>.
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2011 Kim Ryan. All rights reserved.
+Copyright (c) 2016 Kim Ryan. All rights reserved.
 
 This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself, either Perl version 5.8.4 or,
-at your option, any later version of Perl 5 you may have available.
-
-
+it under the same terms as Perl itself.
 
 
 =cut
@@ -39,7 +36,7 @@ package Lingua::EN::NameParse::Grammar;
 use strict;
 use warnings;
 
-our $VERSION = '1.29';
+our $VERSION = '1.34';
 
 
 # Rules that define valid orderings of a names components
@@ -95,6 +92,24 @@ q{
       }
    }
    |
+   
+   title initials conjunction title initials surname non_matching(?)
+   {
+      # Two related people, own initials, shared surname
+      $return =
+      {
+         title_1       => $item[1],
+         initials_1    => $item[2],
+         conjunction_1 => $item[3],
+         title_2       => $item[4],
+         initials_2    => $item[5],
+         surname_1     => $item[6],
+         non_matching  => $item[7][0],
+         number        => 2,
+         type          => 'Mr_A_&_Ms_B_Smith'
+      }
+   }
+   |      
 
    title initials conjunction initials surname non_matching(?)
    {
@@ -113,6 +128,7 @@ q{
       }
    }
    |
+   
 
    title conjunction title initials conjunction initials surname non_matching(?)
    {
@@ -134,23 +150,6 @@ q{
    }
    |
 
-   title initials conjunction title initials surname non_matching(?)
-   {
-      # Two related people, own initials, shared surname
-      $return =
-      {
-         title_1       => $item[1],
-         initials_1    => $item[2],
-         conjunction_1 => $item[3],
-         title_2       => $item[4],
-         initials_2    => $item[5],
-         surname_1     => $item[6],
-         non_matching  => $item[7][0],
-         number        => 2,
-         type          => 'Mr_A_&_Ms_B_Smith'
-      }
-   }
-   |
 
    title conjunction title initials surname non_matching(?)
    {
@@ -169,7 +168,7 @@ q{
    }
    |
 
-   given_name surname conjunction  given_name surname non_matching(?)
+   given_name surname conjunction given_name surname non_matching(?)
    {
       $return =
       {
@@ -185,7 +184,7 @@ q{
    }
    |
 
-   initials surname conjunction  initials surname non_matching(?)
+   initials surname conjunction initials surname non_matching(?)
    {
       $return =
       {
@@ -221,7 +220,7 @@ q{
 my $rules_single_names =
 q{
 
-    precursor(?) title given_name middle_name surname suffix(?) non_matching(?)
+    precursor(?) title given_name_standard middle_name surname suffix(?) non_matching(?)
     {
        $return =
        {
@@ -238,7 +237,7 @@ q{
     }
     |
 
-   precursor(?) title given_name single_initial surname suffix(?) non_matching(?)
+   precursor(?) title given_name_standard single_initial surname suffix(?) non_matching(?)
    {
       $return =
       {
@@ -254,25 +253,6 @@ q{
       }
    }
    |
-   
-    precursor(?) title single_initial middle_name surname suffix(?) non_matching(?)
-    {
-       $return =
-       {
-          precursor     => $item[1][0],
-          title_1       => $item[2],
-          initials_1    => $item[3],
-          middle_name   => $item[4],
-          surname_1     => $item[5],
-          suffix        => $item[6][0],
-          non_matching  => $item[7][0],
-          number        => 1,
-          type          => 'Mr_J_Adam_Smith'
-       }
-    }
-    |   
-   
-
 
    precursor(?) title given_name surname suffix(?) non_matching(?)
    {
@@ -306,7 +286,7 @@ q{
    }
    |
 
-   precursor(?)  given_name_min_2 middle_name surname suffix(?) non_matching(?)
+   precursor(?)  given_name_standard middle_name surname suffix(?) non_matching(?)
    {
       $return =
       {
@@ -322,7 +302,7 @@ q{
    }
    |
 
-   precursor(?) given_name_min_2 single_initial surname suffix(?) non_matching(?)
+   precursor(?) given_name_standard single_initial surname suffix(?) non_matching(?)
    {
       $return =
       {
@@ -337,7 +317,7 @@ q{
       }
    }
    |
-   
+
    precursor(?) single_initial middle_name surname suffix(?) non_matching(?)
    {
       $return =
@@ -352,7 +332,7 @@ q{
          type          => 'J_Adam_Smith'
       }
    }
-   |   
+   |
 
    precursor(?) given_name surname suffix(?) non_matching(?)
    {
@@ -383,8 +363,8 @@ q{
       }
    }
    |
-   
-   given_name non_matching(?)
+
+   given_name_standard non_matching(?)
    {
       $return =
       {
@@ -395,7 +375,7 @@ q{
       }
    }
    |
-
+   
    non_matching(?)
    {
       $return =
@@ -411,300 +391,267 @@ q{
 # Individual components that a name can be composed from. Components are
 # expressed as literals or Perl regular expressions.
 
-my $precursors =
-q
-{
-    precursor : 
-
-    /Estate Of (The Late )?/i |
-    /His (Excellency|Honou?r) /i |
-    /Her (Excellency|Honou?r) /i |
-    /The Right Honou?rable /i |
-    /The Honou?rable /i |
-    /Right Honou?rable /i |
-    /The Rt\.? Hon\.? /i |
-    /The Hon\.? /i |
-    /Rt\.? Hon\.? /i
-
-};
 
 my $titles =
 q{
-
-   title :
-
-   /Mr\.? /i           |
-   /Ms\.? /i           |
-   /M\/s\.? /i         |
-   /Mrs\.? /i          |
-   /Miss\.? /i         |
-
-   /Dr\.? /i           |
-   /Sir /i             |
-   /Dame /i            
-
+    # Place most frequent titles frist to speed matching    
+    title : /(MR|MS|M\/S|MRS|MISS|DR) / 
+ 
 };
-   
+
 my $extended_titles =
 q{
-                       |
-   /Messrs /i          |   # plural or Mr
-   /Mme\.? /i          |   # Madame
-   /Mister /i          |
-   /Mast(\.|er)? /i    |
-   /Ms?gr\.? /i        |   # Monsignor
-   /Count /i           |
-   /Countess /i        |   
-   /Duke /i            |
-   /Duchess /i         |
-   /Lord /i            |
-   /Lady /i            |
-   /Marquess i/        |
-
-   /Madam(e)? /i       |
-
-   # Medical
-   /Doctor /i          |
-   /Sister /i          |
-   /Matron /i          |
-
-   # Legal
-   /Judge /i           |
-   /Justice /i         |
-
-   # Police
-   /Det\.? /i          |
-   /Insp\.? /i         |
-
-   # Military
-   /Brig(adier)? /i       |
-   /Captain /i            |
-   /Capt\.? /i            |
-   /Colonel /i            |
-   /Col\.? /i             |
-   /Commander /i          |
-   /Commodore /i          |
-   /Cdr\.? /i             |   # Commander, Commodore
-   /Field Marshall /i     |
-   /Fl\.? Off\.? /i       |
-   /Flight Officer /i     |
-   /Flt Lt /i             |
-   /Flight Lieutenant /i  |
-   /Gen(\.|eral)? /i      |
-   /Gen\. /i              |
-   /Pte\. /i              |
-   /Private /i            |
-   /Sgt\.? /i             |
-   /Sargent /i            |
-   /Air Commander /i      |
-   /Air Commodore /i      |
-   /Air Marshall /i       |
-   /Lieutenant Colonel /i |
-   /Lt\.? Col\.? /i       |
-   /Lt\.? Gen\.? /i       |
-   /Lt\.? Cdr\.? /i       |
-   /Lieutenant /i         |
-   /(Lt|Leut|Lieut)\.? /i |
-   /Major General /i      |
-   /Maj\.? Gen\.?/i       |
-   /Major /i              |
-   /Maj\.? /i             |
-   /Pilot Officer /i      |
-
-
-   # Religious
-   /Rabbi /i              |
-   /Bishop /i             |
-   /Brother /i            |
-   /Chaplain /i           |
-   /Father /i             |
-   /Pastor /i             |
-   /Mother Superior /i    |
-   /Mother /i             |
-   /Most Rever[e|a]nd /i  |
-   /Very Rever[e|a]nd /i  |
-   /Rever[e|a]nd /i       |
-   /Mt\.? Revd\.? /i      |
-   /V\.? Revd?\.? /i      |
-   /Revd?\.? /i           |
-
-
-   # Other
-   /Prof(\.|essor)? /i    |
-   /Ald(\.|erman)? /i
+    |
+    /(
+    SIR|                     
+    MESSRS| # Plural or Mr
+    MADAME?|
+    MME| # Madame
+    MISTER|
+    MASTER|
+    MAST|
+    MS?GR| # Monsignor
+    COUNT|
+    COUNTESS|
+    DUKE|
+    DUCHESS|
+    LORD|
+    LADY|
+    MARQUESS|
+    
+    # Medical
+    DOCTOR|SISTER|MATRON|
+    
+    # Legal
+    JUDGE|JUSTICE|
+    
+    # Police
+    DET|INSP|CONST|
+    
+    # Military
+    BRIGDIER|BRIG|
+    CAPTAIN|CAPT|
+    COLONEL|COL|
+    COMMANDER IN CHIEF|COMMANDER|
+    COMMODORE|
+    CDR|   # Commander, Commodore
+    FIELD\ MARSHALL|   
+    FLIGHT\ OFFICER| FL OFF|
+    FLIGHT\ LIEUTENANT|FLT LT|
+    PILOT\ OFFICER|
+    GENERAL\ OF\ THE\ ARMY|GENERAL|GEN|
+    PTE|PVT|PRIVATE|
+    SGT|SARGENT|
+    AIR\ COMMANDER|
+    AIR\ COMMODORE|
+    AIR\ MARSHALL|
+    LIEUTENANT\ COLONEL|LT\ COL|
+    LT\ GEN|
+    LT\ CDR|
+    LIEUTENANT|LT|LEUT|LIEUT|
+    MAJOR GENERAL|MAJ GEN|
+    MAJOR|MAJ|
+    
+    # Religious
+    RABBI|
+    BISHOP|
+    BROTHER|
+    CHAPLAIN|
+    FATHER|
+    PASTOR|
+    MOTHER\ SUPERIOR|MOTHER|
+    MOST\ REVER[E|A]ND|
+    MUFTI|
+    VERY\ REVER[E|A]ND|
+    REVER[E|A]ND|
+    MT\ REVD|V\ REVD|REVD|
+    
+    # Other
+    PROFESSOR|
+    PROF|
+    ALDERMAN|ALD   
+    )\ /x
 };
 
-my $conjunction = q{ conjunction : /And |& /i };
+my $common =
+q{
 
-# Used in the John_A_Smith and J_Adam_Smith name types. Although this 
-# duplicates $initials_1, it is needed because this type of initial must 
-# always be one character long, regardless of the length of initials set 
-# by the user in the 'new' method.
-my $single_initial = q{ single_initial: /[A-Z]\.? /i };
+    precursor :
+        /(
+        ESTATE\ OF\ THE\ LATE|
+        ESTATE\ OF|
+        HIS\ EXCELLENCY|
+        HIS\ HONOU?R|
+        HER\ EXCELLENCY|
+        HER\ HONOU?R|
+        THE\ RIGHT HONOU?RABLE|
+        THE\ HONOU?RABLE|
+        RIGHT\ HONOU?RABLE|
+        THE\ RT\ HON|
+        THE\ HON|
+        RT\ HON    
+        )\ /x
+    
+    conjunction : /AND |& /
+
+    # Used in the John_A_Smith and J_Adam_Smith name types, as well as when intials are set to 1
+    single_initial: /[A-Z] /
+
+    # Examples are Jo-Anne, D'Artagnan, O'Shaugnessy La'Keishia, T-Bone
+    split_given_name :  /[A-Z]{1,}['|-][A-Z]{2,} /
+
+    constonant: /[A-DF-HJ-NP-TV-Z]]/
+    
+    # For use with John_Adam_Smith and John_A_Smith name types
+    given_name_standard:
+        /[A-Z]{3,} / |
+        /[AEIOU]/ constonant / / |
+        constonant /[AEIOUY] / |
+        split_given_name
+    
+   # Patronymic, place name and other surname prefixes
+    prefix:
+    /(
+        [A|E]L|   # ARABIC, GREEK,
+        AP|       # WELSH
+        BEN|      # HEBREW
+        
+        DELLA|DELLE|DALLE|   # ITALIAN               
+        DELA|
+        DELL?|
+        DE\ LA|
+        DE\ LOS|
+        DE|
+        D[A|I|U]|
+        L[A|E|O]|
+        
+        ST|       # ABBREVIATION FOR SAINT
+        SAN|      # SPANISH
+        
+        # DUTCH
+        DEN|     
+        VON\ DER|
+        VON|
+        VAN\ DE[N|R]|
+        VAN
+    )\ /x
+    |
+    /[D|L|O]'/ # ITALIAN, IRISH OR FRENCH, abbreviation for 'the', 'of' etc
+    |
+    /D[A|E]LL'/  
+    
+    middle_name:
+    
+    # Dont grab surname prefix too early. For example, John Van Dam could be
+    # interpreted as middle name of Van and Surname of Dam. So exclude prefixs
+    # from middle names
+    ...!prefix given_name
+    {
+       $return = $item[2];
+    }
+
+
+    # Use look-ahead to avoid ambiguity between surname and suffix. For example,
+    # John Smith Snr, would detect Snr as the surname and Smith as the middle name
+    surname : ...!suffix first_surname second_surname(?)
+    {
+       if ( $item[2] and $item[3][0] )
+       {
+          $return = "$item[2]$item[3][0]";
+       }
+       else
+       {
+          $return = $item[2];
+       }
+    }
+    
+    first_surname : prefix name
+    {
+       $return = "$item[1]$item[2]";
+    }
+    |
+    name
+
+
+    second_surname : '-' name
+    {
+       if ( $item[1] and $item[2] )
+       {
+          $return = "$item[1]$item[2]";
+       }
+    }
+   
+   # Note space will not occur for first part of a hphenated surname
+   # AddressParse::_valid_name will do further check on name context 
+    name : /[A-Z]{2,} ?/  
+
+  
+   suffix:
+
+    /(
+    ESQUIRE|
+    ESQ |
+    SN?R| # Senior
+    JN?R| # Junior
+    PHD |
+    MD  |
+    LLB |
+
+    XI{1,3}| # 11th, 12th, 13th
+    X       | # 10th
+    IV      | # 4th
+    VI{1,3} | # 6th, 7th, 8th
+    V       | # 5th
+    IX      | # 9th
+    I{1,3}     # 1st, 2nd, 3rd
+    )\ /x  
+
+
+    # One or more characters. 
+    non_matching: /.*/     
+};
 
 # Define given name combinations, specifying the minimum number of letters.
 # The correct pair of rules is determined by the 'initials' key in the hash
 # passed to the 'new' method.
 
-# Examples are Jo, Jo-Anne, D'Artagnan, O'Shaugnessy La'Keishia, T-Bone
-my $given_name_min_2 =
-q{
-    given_name: /[A-Z]{2,} /i | /[A-Z]{2,}\-[A-Z]{2,} /i | /[A-Z]{1,}\'[A-Z]{2,} /i | /T\-Bone /i
-};
+
+my $given_name_min_2 = q{ given_name : given_name_standard  };
 
 # Joe, Jo-Anne ...
 my $given_name_min_3 =
 q{
-    given_name: /[A-Z]{3,} /i | /[A-Z]{2,}\-[A-Z]{2,} /i | /[A-Z]{1,}\'[A-Z]{2,} /i | /T\-Bone /i
+    given_name: /[A-Z]{3,} / | split_given_name
 };
 
 
 # John ...
 my $given_name_min_4 =
 q{
-    given_name: /[A-Z]{4,} /i | /[A-Z]{2,}\-[A-Z]{2,} /i | /[A-Z]{1,}\'[A-Z]{3,} /i | /T\-Bone /i
-};
-
-# For use with John_Adam_Smith and John_A_Smith name types
-my $fixed_length_given_name =
-q{
-    given_name_min_2 : /[A-Z]{2,} /i | /[A-Z]{2,}\-[A-Z]{2,} /i | /[A-Z]{1,}\'[A-Z]{2,} /i
+    given_name: /[A-Z]{4,} / | split_given_name
 };
 
 
 # Define initials combinations specifying the minimum and maximum letters.
 # Order from most complex to simplest,  to avoid premature matching.
 
-# 'A' 'A.'
-my $initials_1 = q{ initials: /[A-Z]\.? /i };
+# 'A'
+my $initials_1 = q{ initials : single_initial };
 
-# 'A. B.' 'A.B.' 'AB' 'A B'
+#'AB' 'A B'
 
 my $initials_2 =
 q{
-   initials:  /([A-Z]\. ){1,2}/i | /([A-Z]\.){1,2} /i | /([A-Z] ){1,2}/i | /([A-Z]){1,2} /i
+   initials:  /([A-Z] ){1,2}/ | /([A-Z]){1,2} /
 };
 
-# 'A. B. C. '  'A.B.C' 'ABC' 'A B C'
+# 'ABC' or 'A B C'
 my $initials_3 =
 q{
-   initials: /([A-Z]\. ){1,3}/i |  /([A-Z]\.){1,3} /i | /([A-Z] ){1,3}/i | /([A-Z]){1,3} /i
+   initials: /([A-Z] ){1,3}/ | /([A-Z]){1,3} /
 };
 
 
-# Jo, Jo-Anne, La'Keishia, D'Artagnan, O'Shaugnessy 
-my $middle_name =
-q{
-   middle_name: 
-   
-   # Dont grab surname prefix too early. For example, John Van Dam could be
-   # interpreted as middle name of Van and Surname of Dam. So exclude prefixs
-   # from middle names
-   ...!prefix /[A-Z]{2,} /i | /[A-Z]{2,}\-[A-Z]{2,} /i | /[A-Z]{1,}\'[A-Z]{2,} /i
-   {
-      $return = $item[2];
-   }
-};
-
-
-my $full_surname =
-q{
-   # Use look-ahead to avoid ambiguity between surname and suffix. For example,
-   # John Smith Snr, would detect Snr as the surname and Smith as the middle name
-   surname : ...!suffix sub_surname second_name(?)
-   {
-      if ( $item[2] and $item[3][0] )
-      {
-         $return = "$item[2]$item[3][0]";
-      }
-      else
-      {
-         $return = $item[2];
-      }
-   }
-
-   sub_surname : prefix(?) name
-   {
-      # To prevent warnings when compiling with the -w switch,
-      # do not return uninitialized variables.
-      if ( $item[1][0] )
-      {
-         $return = "$item[1][0]$item[2]";
-      }
-      else
-      {
-         $return = $item[2];
-      }
-   }
-
-   second_name : '-' sub_surname
-   {
-      if ( $item[1] and $item[2] )
-      {
-         $return = "$item[1]$item[2]";
-      }
-   }
-
-   # Patronymic, place name and other surname prefixes
-   prefix:
-
-      /[A|E]l /i         |   # Arabic, Greek,
-      /Ap /i             |   # Welsh
-      /Ben /i            |   # Hebrew
-
-      /Dell([a|e])? /i   |   # ITALIAN
-      /Dalle /i          |
-      /D[a|e]ll'/i       |
-      /Dela /i           |
-      /Del /i            |
-      /De (La |Los )?/i  |
-      /D[a|i|u] /i       |
-      /L[a|e|o] /i       |
-
-      /[D|L|O]'/i        |   # Italian, Irish or French
-      /St\.? /i          |   # abbreviation for Saint
-      /San /i            |   # Spanish
-
-      /Den /i            |   # DUTCH
-      /Von (Der )?/i     |
-      /Van (De(n|r)? )?/i
-
-   # space needed for any following text
-   name: /[A-Z]{2,} ?/i
-
-};
-
-my $suffix =
-q{
-   suffix:
-
-      # word boundaries are used to stop partial matches from surnames such as 
-      # the "VI" in "VINCE"
-
-      /Esq(\.|uire)?\b ?/i |
-      /Sn?r\.?\b ?/i | # Senior
-      /Jn?r\.?\b ?/i | # Junior
-      /PhD\.?\b ?/i  | 
-      /MD\.?\b ?/i   | 
-      /LLB\.?\b ?/i  | 
-
-
-      /XI{1,3}\b ?/i | # 11th, 12th, 13th
-      /X\b ?/i       | # 10th
-      /IV\b ?/i      | # 4th
-      /VI{1,3}\b ?/i | # 6th, 7th, 8th
-      /V\b ?/i       | # 5th
-      /IX\b ?/i      | # 9th
-      /I{1,3}\b ?/i    # 1st, 2nd, 3rd
-};
-
-# Two or more characters. This is set to 2 as a work around for the problem
-# with detecting suffixes like Snr. and Jnr. The dot here gets picked up
-# as non matching.
-
-my $non_matching = q{ non_matching: /.{2,}/ };
-
-
 #-------------------------------------------------------------------------------
 # Assemble correct combination for grammar tree.
 
@@ -713,22 +660,24 @@ sub _create
    my $name = shift;
 
    my $grammar = $rules_start;
+   
 
    if ( $name->{joint_names} )
    {
        $grammar .= $rules_joint_names;
-   }    
-   $grammar .= $rules_single_names . $precursors . $titles;
+   }
+   $grammar .= $rules_single_names;
+   
+   
+   $grammar .= $common;
+   
+   $grammar .= $titles;
 
     if ( $name->{extended_titles} )
     {
         $grammar .= $extended_titles;
     }
 
-   $grammar .= $conjunction;
-
-   $grammar .= $single_initial;
-
    $name->{initials} > 3 and $name->{initials} = 3;
    $name->{initials} < 1 and $name->{initials} = 1;
 
@@ -736,6 +685,8 @@ sub _create
    # a given name. For example, if initials are set to 2, MR TO SMITH
    # will have initials of T & O and no given name, but MR TOM SMITH will
    # have no initials, and a given name of Tom.
+   
+
 
    if ( $name->{initials} == 1 )
    {
@@ -743,20 +694,14 @@ sub _create
    }
    elsif ( $name->{initials} == 2 )
    {
-      $grammar .= $given_name_min_3 . $initials_2;
+      $grammar .=  $initials_2 . $given_name_min_3;
    }
    elsif ( $name->{initials} == 3 )
    {
       $grammar .= $given_name_min_4 . $initials_3;
    }
-   
-   $grammar .= $fixed_length_given_name
-             . $middle_name
-             . $full_surname
-             . $suffix
-             . $non_matching
-             ;
 
+ 
    return($grammar);
 }
 #-------------------------------------------------------------------------------
